@@ -36,7 +36,12 @@
 #include <glib/gi18n.h>
 #include <libnautilus-private/nautilus-icon-factory.h>
 #include <libnautilus-private/nautilus-dnd.h>
+
+#if GLIB_CHECK_VERSION (2, 13, 0)
+#include <glib/gsequence.h>
+#else
 #include <gsequence/gsequence.h>
+#endif
 
 enum {
 	SUBDIRECTORY_UNLOADED,
@@ -942,6 +947,11 @@ fm_list_model_add_file (FMListModel *model, NautilusFile *file,
 
 	if (parent_ptr != NULL) {
 		file_entry->parent = g_sequence_get (parent_ptr);
+		/* At this point we set loaded. Either we saw
+		 * "done" and ignored it waiting for this, or we do this
+		 * earlier, but then we replace the dummy row anyway,
+		 * so it doesn't matter */
+		file_entry->parent->loaded = 1;
 		parent_hash = file_entry->parent->reverse_map;
 		files = file_entry->parent->files;
 		if (g_sequence_get_length (files) == 1) {
@@ -1619,7 +1629,7 @@ fm_list_model_get_type (void)
 	static GType object_type = 0;
 
 	if (object_type == 0) {
-		static const GTypeInfo object_info = {
+		const GTypeInfo object_info = {
 			sizeof (FMListModelClass),
 			NULL,		/* base_init */
 			NULL,		/* base_finalize */
@@ -1631,19 +1641,19 @@ fm_list_model_get_type (void)
 			(GInstanceInitFunc) fm_list_model_init,
 		};
 
-		static const GInterfaceInfo tree_model_info = {
+		const GInterfaceInfo tree_model_info = {
 			(GInterfaceInitFunc) fm_list_model_tree_model_init,
 			NULL,
 			NULL
 		};
 
-		static const GInterfaceInfo sortable_info = {
+		const GInterfaceInfo sortable_info = {
 			(GInterfaceInitFunc) fm_list_model_sortable_init,
 			NULL,
 			NULL
 		};
 
-		static const GInterfaceInfo multi_drag_source_info = {
+		const GInterfaceInfo multi_drag_source_info = {
 			(GInterfaceInitFunc) fm_list_model_multi_drag_source_init,
 			NULL,
 			NULL
@@ -1664,78 +1674,44 @@ fm_list_model_get_type (void)
 	return object_type;
 }
 
-struct ChangeDummyData {
-	FMListModel *model;
-	NautilusDirectory *directory;
-};
-
-static gboolean
-change_dummy_row_callback (gpointer callback_data)
+void
+fm_list_model_subdirectory_done_loading (FMListModel *model, NautilusDirectory *directory)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	FileEntry *file_entry, *dummy_entry;
 	GSequenceIter *parent_ptr, *dummy_ptr;
-	FMListModel *model;
 	GSequence *files;
 	
-	struct ChangeDummyData *data;
-	data = callback_data;
-
-	if (data->model != NULL) {
-		model = data->model;
-		
-		parent_ptr = g_hash_table_lookup (model->details->directory_reverse_map,
-						  data->directory);
-		file_entry = g_sequence_get (parent_ptr);
-
-		file_entry->loaded = 1;
-		files = file_entry->files;
-	
-		if (g_sequence_get_length (files) == 1) {
-			dummy_ptr = g_sequence_get_iter_at_pos (file_entry->files, 0);
-			dummy_entry = g_sequence_get (dummy_ptr);
-			if (dummy_entry->file == NULL) {
-				/* was the dummy file */
-				
-				iter.stamp = model->details->stamp;
-				iter.user_data = dummy_ptr;
-				
-				path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
-				gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
-				gtk_tree_path_free (path);
-			}
-		}
+	if (model == NULL || model->details->directory_reverse_map == NULL) {
+		return;
 	}
-
-	eel_remove_weak_pointer (&data->model);
-	nautilus_directory_unref (data->directory);
-	g_free (data);
-	
-	return FALSE;
-}
-
-void
-fm_list_model_subdirectory_done_loading (FMListModel *model, NautilusDirectory *directory)
-{
-	GSequenceIter *parent_ptr;
-	struct ChangeDummyData *data;
-
 	parent_ptr = g_hash_table_lookup (model->details->directory_reverse_map,
 					  directory);
 	if (parent_ptr == NULL) {
 		return;
 	}
+	
+	file_entry = g_sequence_get (parent_ptr);
+	files = file_entry->files;
 
-	/* Delay slightly before sending the change even for the dummy
-	 * row, to avoid flicker from loading -> empty -> file if we
-	 * get files added quickly.
+	/* Only swap loading -> empty if we saw no files yet at "done",
+	 * otherwise, toggle loading at first added file to the model.
 	 */
-	data = g_new (struct ChangeDummyData, 1);
-	data->model = model;
-	data->directory = directory;
-	nautilus_directory_ref (directory);
-	eel_add_weak_pointer (&data->model);
-
-	g_timeout_add (LOADING_TO_EMPTY_DELAY, change_dummy_row_callback, data);
+	if (!nautilus_directory_is_not_empty (directory) &&
+	    g_sequence_get_length (files) == 1) {
+		dummy_ptr = g_sequence_get_iter_at_pos (file_entry->files, 0);
+		dummy_entry = g_sequence_get (dummy_ptr);
+		if (dummy_entry->file == NULL) {
+			/* was the dummy file */
+			file_entry->loaded = 1;
+			
+			iter.stamp = model->details->stamp;
+			iter.user_data = dummy_ptr;
+			
+			path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+			gtk_tree_path_free (path);
+		}
+	}
 }
