@@ -35,6 +35,7 @@
 #include "nautilus-pathbar.h"
 #include "nautilus-main.h"
 #include "nautilus-window-private.h"
+#include "nautilus-trash-bar.h"
 #include "nautilus-zoom-control.h"
 #include <eel/eel-accessibility.h>
 #include <eel/eel-debug.h>
@@ -54,6 +55,7 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-extension/nautilus-location-widget-provider.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file.h>
@@ -63,6 +65,7 @@
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-monitor.h>
 #include <libnautilus-private/nautilus-search-directory.h>
+#include <libnautilus-private/nautilus-trash-directory.h>
 #include <libnautilus-private/nautilus-view-factory.h>
 #include <libnautilus-private/nautilus-window-info.h>
 
@@ -330,12 +333,32 @@ viewed_file_changed_callback (NautilusFile *file,
                          * all other nautilus_window_close callers?
                          */
                         end_location_change (window);
-                        
-                        /* FIXME bugzilla.gnome.org 45038: Is closing
-                         * the window really the right thing to do for
-                         * all cases?
-                         */
-                        nautilus_window_close (window);
+
+			if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
+				/* auto-show existing parent URI. */
+				char *uri, *go_to_uri;
+				NautilusFile *parent_file;
+
+				parent_file = nautilus_file_get_parent (file);
+				uri = nautilus_file_get_uri (parent_file);
+
+				go_to_uri = nautilus_find_existing_uri_in_hierarchy (uri);
+				if (go_to_uri != NULL) {
+					/* the path bar URI will be set to go_to_uri immediately
+					 * in begin_location_change, but we don't want the
+					 * inexistant children to show up anymore */
+					nautilus_path_bar_clear_buttons (NAUTILUS_PATH_BAR (NAUTILUS_NAVIGATION_WINDOW (window)->path_bar));
+					nautilus_window_go_to (NAUTILUS_WINDOW (window), go_to_uri);
+					g_free (go_to_uri);
+				} else {
+					nautilus_window_go_home (NAUTILUS_WINDOW (window));
+				}
+
+				g_free (uri);
+				nautilus_file_unref (parent_file);
+			} else {
+				nautilus_window_close (window);
+			}
                 }
 	} else {
                 new_location = nautilus_file_get_uri (file);
@@ -453,9 +476,18 @@ nautilus_window_open_location_full (NautilusWindow *window,
 {
         NautilusWindow *target_window;
         gboolean do_load_location = TRUE;
+	char *old_location;
         
         target_window = NULL;
 
+	old_location = nautilus_window_get_location (window);
+	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "window %p open location: old=\"%s\", new=\"%s\"",
+			    window,
+			    old_location ? old_location : "(none)",
+			    location);
+	g_free (old_location);
+
 	switch (mode) {
         case NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE :
 		if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER)) {
@@ -657,23 +689,6 @@ nautilus_window_content_view_matches_iid (NautilusWindow *window,
 }
 
 
-static gboolean
-another_navigation_window_already_showing (NautilusWindow *the_window)
-{
-	GList *list, *item;
-	
-	list = nautilus_application_get_window_list ();
-	for (item = list; item != NULL; item = item->next) {
-		if (item->data != the_window &&
-		    NAUTILUS_IS_NAVIGATION_WINDOW (item->data)) {
-			return TRUE;
-		}
-	}
-	
-	return FALSE;
-}
-
-
 /*
  * begin_location_change
  * 
@@ -783,6 +798,7 @@ setup_new_window (NautilusWindow *window, NautilusFile *file)
 	char *show_hidden_file_setting;
 	char *geometry_string;
 	char *scroll_string;
+	gboolean maximized;
 	
 	if (NAUTILUS_IS_SPATIAL_WINDOW (window) && !NAUTILUS_IS_DESKTOP_WINDOW (window)) {
 		/* load show hidden state */
@@ -801,6 +817,14 @@ setup_new_window (NautilusWindow *window, NautilusFile *file)
 		g_free (show_hidden_file_setting);
 		
 		/* load the saved window geometry */
+		maximized = nautilus_file_get_boolean_metadata
+			(file, NAUTILUS_METADATA_KEY_WINDOW_MAXIMIZED, FALSE);
+		if (maximized) {
+			gtk_window_maximize (GTK_WINDOW (window));
+		} else {
+			gtk_window_unmaximize (GTK_WINDOW (window));
+		}
+
 		geometry_string = nautilus_file_get_metadata 
 			(file, NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY, NULL);
                 if (geometry_string != NULL) {
@@ -829,25 +853,6 @@ setup_new_window (NautilusWindow *window, NautilusFile *file)
 			window->details->pending_scroll_to = scroll_string;
 		}
         }
-	
-        if (NAUTILUS_IS_NAVIGATION_WINDOW (window)) {
-		geometry_string = eel_preferences_get
-				(NAUTILUS_PREFERENCES_NAVIGATION_WINDOW_SAVED_GEOMETRY);
-                if (geometry_string != NULL &&
-		    geometry_string[0] != 0) {
-			/* Ignore saved window position if a window with the same
-			 * location is already showing. That way the two windows
-			 * wont appear at the exact same location on the screen.
-			 */
-                        eel_gtk_window_set_initial_geometry_from_string 
-                                (GTK_WINDOW (window), 
-                                 geometry_string,
-                                 NAUTILUS_WINDOW_MIN_WIDTH, 
-                                 NAUTILUS_WINDOW_MIN_HEIGHT,
-				 another_navigation_window_already_showing (window));
-                }
-                g_free (geometry_string);
-	}
 }
 
 static void
@@ -939,7 +944,7 @@ got_file_info_for_view_selection_callback (NautilusFile *file,
 				   better test for that */
 				
 				if (!gnome_vfs_uris_match (location, "file:///")) {
-					home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+					home_uri = nautilus_get_home_directory_uri ();
 					if (!gnome_vfs_uris_match (home_uri, location)) {	
 						nautilus_window_go_home (NAUTILUS_WINDOW (window));
 					} else {
@@ -1170,7 +1175,19 @@ add_extension_extra_widgets (NautilusWindow *window, const char *uri)
 	}
 
 	nautilus_module_extension_list_free (providers);
-	
+}
+
+static void
+nautilus_window_show_trash_bar (NautilusWindow *window)
+{
+	GtkWidget *bar;
+
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	bar = nautilus_trash_bar_new ();
+	gtk_widget_show (bar);
+
+	nautilus_window_add_extra_location_widget (window, bar);
 }
 
 /* Handle the changes for the NautilusWindow itself. */
@@ -1230,6 +1247,11 @@ update_for_new_location (NautilusWindow *window)
 		} else {
 			nautilus_window_set_search_mode (window, FALSE, NULL);
 		}
+
+		if (NAUTILUS_IS_TRASH_DIRECTORY (directory)) {
+			nautilus_window_show_trash_bar (window);
+		}
+
 		nautilus_directory_unref (directory);
 
 		add_extension_extra_widgets (window, window->details->location);
@@ -1282,6 +1304,15 @@ nautilus_window_report_load_complete (NautilusWindow *window,
 static void
 end_location_change (NautilusWindow *window)
 {
+	char *location;
+
+	location = nautilus_window_get_location (window);
+	if (location) {
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "finished loading window %p: %s", window, location);
+		g_free (location);
+	}
+
         nautilus_window_allow_stop (window, FALSE);
 
         /* Now we can free pending_scroll_to, since the load_complete
@@ -1539,11 +1570,18 @@ nautilus_window_set_content_view (NautilusWindow *window,
                                   const char *id)
 {
 	NautilusFile *file;
+	char *location;
 	
 	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
         g_return_if_fail (window->details->location != NULL);
 	g_return_if_fail (id != NULL);
 
+	location = nautilus_window_get_location (window);
+	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "change view of window %p: \"%s\" to \"%s\"",
+			    window, location, id);
+	g_free (location);
+
         if (nautilus_window_content_view_matches_iid (window, id)) {
         	return;
         }
